@@ -19,6 +19,7 @@ from twisted.internet import defer
 from twisted.python import failure
 
 from buildbot.data import connector
+from buildbot.data import resultspec
 from buildbot.db.buildrequests import AlreadyClaimedError
 from buildbot.test.util import validation
 from buildbot.util import service
@@ -40,7 +41,7 @@ class FakeUpdates(service.AsyncService):
         self.maybeBuildsetCompleteCalls = 0
         self.masterStateChanges = []  # dictionaries
         self.schedulerIds = {}  # { name : id }; users can add schedulers here
-        self.builderIds = {}  # { name : id }; users can add schedulers here
+        self.builderIds = {}  # { name : id }; users can add builders here
         self.schedulerMasters = {}  # { schedulerid : masterid }
         self.changesourceMasters = {}  # { changesourceid : masterid }
         self.workerIds = {}  # { name : id }; users can add workers here
@@ -67,11 +68,11 @@ class FakeUpdates(service.AsyncService):
             try:
                 json.dumps(propval)
             except (TypeError, ValueError):
-                self.testcase.fail("value for %s is not JSON-able" % (k,))
+                self.testcase.fail(f"value for {k} is not JSON-able")
 
     # update methods
 
-    def addChange(self, files=None, comments=None, author=None,
+    def addChange(self, files=None, comments=None, author=None, committer=None,
                   revision=None, when_timestamp=None, branch=None, category=None,
                   revlink='', properties=None, repository='', codebase=None,
                   project='', src=None):
@@ -84,12 +85,14 @@ class FakeUpdates(service.AsyncService):
             map(lambda f: self.testcase.assertIsInstance(f, str), files)
         self.testcase.assertIsInstance(comments, (type(None), str))
         self.testcase.assertIsInstance(author, (type(None), str))
+        self.testcase.assertIsInstance(committer, (type(None), str))
         self.testcase.assertIsInstance(revision, (type(None), str))
         self.testcase.assertIsInstance(when_timestamp, (type(None), int))
         self.testcase.assertIsInstance(branch, (type(None), str))
 
         if callable(category):
             pre_change = self.master.config.preChangeGenerator(author=author,
+                                                               committer=committer,
                                                                files=files,
                                                                comments=comments,
                                                                revision=revision,
@@ -135,7 +138,7 @@ class FakeUpdates(service.AsyncService):
     @defer.inlineCallbacks
     def addBuildset(self, waited_for, scheduler=None, sourcestamps=None, reason='',
                     properties=None, builderids=None, external_idstring=None,
-                    parent_buildid=None, parent_relationship=None):
+                    parent_buildid=None, parent_relationship=None, priority=0):
         if sourcestamps is None:
             sourcestamps = []
         if properties is None:
@@ -147,8 +150,7 @@ class FakeUpdates(service.AsyncService):
         self.testcase.assertIsInstance(sourcestamps, list)
         for ss in sourcestamps:
             if not isinstance(ss, int) and not isinstance(ss, dict):
-                self.testcase.fail("%s (%s) is not an integer or a dictionary"
-                                   % (ss, type(ss)))
+                self.testcase.fail(f"{ss} ({type(ss)}) is not an integer or a dictionary")
             del ss  # since we use locals(), below
         self.testcase.assertIsInstance(reason, str)
         self.assertProperties(sourced=True, properties=properties)
@@ -208,6 +210,28 @@ class FakeUpdates(service.AsyncService):
     def rebuildBuildrequest(self, buildrequest):
         return defer.succeed(None)
 
+    @defer.inlineCallbacks
+    def update_project_info(
+        self,
+        projectid,
+        slug,
+        description,
+        description_format,
+        description_html,
+    ):
+        yield self.master.db.projects.update_project_info(
+            projectid,
+            slug,
+            description,
+            description_format,
+            description_html
+        )
+
+    def find_project_id(self, name):
+        validation.verifyType(self.testcase, 'project name', name,
+                              validation.StringValidator())
+        return self.master.db.projects.find_project_id(name)
+
     def updateBuilderList(self, masterid, builderNames):
         self.testcase.assertEqual(masterid, self.master.masterid)
         for n in builderNames:
@@ -215,8 +239,11 @@ class FakeUpdates(service.AsyncService):
         self.builderNames = builderNames
         return defer.succeed(None)
 
-    def updateBuilderInfo(self, builderid, description, tags):
-        yield self.master.db.builders.updateBuilderInfo(builderid, description, tags)
+    @defer.inlineCallbacks
+    def updateBuilderInfo(self, builderid, description, description_format, description_html,
+                          projectid, tags):
+        yield self.master.db.builders.updateBuilderInfo(builderid, description, description_format,
+                                                        description_html, projectid, tags)
 
     def masterDeactivated(self, masterid):
         return defer.succeed(None)
@@ -301,7 +328,7 @@ class FakeUpdates(service.AsyncService):
         try:
             json.dumps(value)
         except (TypeError, ValueError):
-            self.testcase.fail("Value for %s is not JSON-able" % name)
+            self.testcase.fail(f"Value for {name} is not JSON-able")
         validation.verifyType(self.testcase, 'source', source,
                               validation.StringValidator())
         return defer.succeed(None)
@@ -359,8 +386,7 @@ class FakeUpdates(service.AsyncService):
         validation.verifyType(self.testcase, 'type', type,
                               validation.IdentifierValidator(1))
         logid = max([0] + list(self.logs)) + 1
-        self.logs[logid] = dict(
-            name=name, type=type, content=[], finished=False)
+        self.logs[logid] = {"name": name, "type": type, "content": [], "finished": False}
         return defer.succeed(logid)
 
     def finishLog(self, logid):
@@ -423,6 +449,50 @@ class FakeUpdates(service.AsyncService):
             paused=paused,
             graceful=graceful)
 
+    # methods form BuildData resource
+    @defer.inlineCallbacks
+    def setBuildData(self, buildid, name, value, source):
+        validation.verifyType(self.testcase, 'buildid', buildid, validation.IntValidator())
+        validation.verifyType(self.testcase, 'name', name, validation.StringValidator())
+        validation.verifyType(self.testcase, 'value', value, validation.BinaryValidator())
+        validation.verifyType(self.testcase, 'source', source, validation.StringValidator())
+        yield self.master.db.build_data.setBuildData(buildid, name, value, source)
+
+    # methods from TestResultSet resource
+    @defer.inlineCallbacks
+    def addTestResultSet(self, builderid, buildid, stepid, description, category, value_unit):
+        validation.verifyType(self.testcase, 'builderid', builderid, validation.IntValidator())
+        validation.verifyType(self.testcase, 'buildid', buildid, validation.IntValidator())
+        validation.verifyType(self.testcase, 'stepid', stepid, validation.IntValidator())
+        validation.verifyType(self.testcase, 'description', description,
+                              validation.StringValidator())
+        validation.verifyType(self.testcase, 'category', category, validation.StringValidator())
+        validation.verifyType(self.testcase, 'value_unit', value_unit, validation.StringValidator())
+
+        test_result_setid = \
+            yield self.master.db.test_result_sets.addTestResultSet(builderid, buildid, stepid,
+                                                                   description, category,
+                                                                   value_unit)
+        return test_result_setid
+
+    @defer.inlineCallbacks
+    def completeTestResultSet(self, test_result_setid, tests_passed=None, tests_failed=None):
+        validation.verifyType(self.testcase, 'test_result_setid', test_result_setid,
+                              validation.IntValidator())
+        validation.verifyType(self.testcase, 'tests_passed', tests_passed,
+                              validation.NoneOk(validation.IntValidator()))
+        validation.verifyType(self.testcase, 'tests_failed', tests_failed,
+                              validation.NoneOk(validation.IntValidator()))
+
+        yield self.master.db.test_result_sets.completeTestResultSet(test_result_setid,
+                                                                    tests_passed, tests_failed)
+
+    # methods from TestResult resource
+    @defer.inlineCallbacks
+    def addTestResults(self, builderid, test_result_setid, result_values):
+        yield self.master.db.test_results.addTestResults(builderid, test_result_setid,
+                                                         result_values)
+
 
 class FakeDataConnector(service.AsyncMultiService):
     # FakeDataConnector delegates to the real DataConnector so it can get all
@@ -440,6 +510,7 @@ class FakeDataConnector(service.AsyncMultiService):
         self.realConnector = connector.DataConnector()
         self.realConnector.setServiceParent(self)
         self.rtypes = self.realConnector.rtypes
+        self.plural_rtypes = self.realConnector.plural_rtypes
 
     def _scanModule(self, mod):
         return self.realConnector._scanModule(mod)
@@ -459,7 +530,20 @@ class FakeDataConnector(service.AsyncMultiService):
         return self.realConnector.get(path, filters=filters, fields=fields,
                                       order=order, limit=limit, offset=offset)
 
+    def get_with_resultspec(self, path, rspec):
+        if not isinstance(path, tuple):
+            raise TypeError('path must be a tuple')
+        if not isinstance(rspec, resultspec.ResultSpec):
+            raise TypeError('rspec must be ResultSpec')
+        return self.realConnector.get_with_resultspec(path, rspec)
+
     def control(self, action, args, path):
         if not isinstance(path, tuple):
             raise TypeError('path must be a tuple')
         return self.realConnector.control(action, args, path)
+
+    def resultspec_from_jsonapi(self, args, entityType, is_collection):
+        return self.realConnector.resultspec_from_jsonapi(args, entityType, is_collection)
+
+    def getResourceTypeForGraphQlType(self, type):
+        return self.realConnector.getResourceTypeForGraphQlType(type)
